function [p stat] = indtestpopt(x, numLevels, ~, j1, j2, j3, TestParam, ~, ~)
%INDTESTPOPT p-value of generic test of conditional independence
%   P = INDTESTPOPT(X, NUMLEVELS, ORD, J1, J2, J3, TESTPARAM, POWERPARAM, PARAM)
%   returns the p-value P of the generic test of conditional independence
%   performed by 
%   ORG.MENSXMACHINA.COMMONS.STATS.INDTESTOPT(X, NUMLEVELS, J1, J2, J3, CHI2INDTESTPARAM, FISHERZINDTESTPARAM)
%   with CHI2INDTESTPARAM = 
%   struct('Alpha', NaN, 'Stat', TESTPARAM.Chi2IndTestParam.Stat, 'Obs', [], 'DoF', [])
%   and FISHERZINDTESTPARAM = struct('Alpha', NaN). ORD, POWERPARAM and
%   PARAM are not used.
%
%   INDTESTPOPT is used as PFUN in constraint-based learning
%   functions in conjunction with
%   ORG.MENSXMACHINA.PGM.INDTESTPOWER.INDTESTPWROPT as POWERFUN and 
%   ORG.MENSXMACHINA.PGM.INDTESTPOWERMAXK.INDTESTPWRMAXKOPT as POWERMAXKFUN.
%
%   [P, STAT] = INDTESTPOPT(X, NUMLEVELS, ORD, J1, J2, J3, TESTPARAM, POWERPARAM, PARAM)
%   also returns the test statistic STAT.
%
%   See also ORG.MENSXMACHINA.PGM.INDTESTPOWER.INDTESTPWROPT, 
%   ORG.MENSXMACHINA.PGM.INDTESTPOWERMAXK.INDTESTPWRMAXKOPT

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

persistent Chi2IndTestParam FisherZIndTestParam;

import org.mensxmachina.commons.stats.*;

if isempty(Chi2IndTestParam)
    Chi2IndTestParam = struct('Alpha', NaN, 'Stat', NaN, [], 'Obs', [], 'DoF', []);
    FisherZIndTestParam = struct('Alpha', NaN);
end

% update stat, leave alpha NaN
Chi2IndTestParam.Stat = TestParam.Chi2IndTestParam.Stat;

[~, p, stat] = indtestopt(x, numLevels, j1, j2, j3, Chi2IndTestParam, FisherZIndTestParam);

end