function [power PowerPParam] = chi2indtesthspopt(x, numLevels, ~, j1, j2, j3, ~, ~, Param)
%CHI2INDTESTHSPOPT Heuristic size power of chi-square conditional independence test (optimized)
%   POWER = CHI2INDTESTHSPOPT(X, NUMLEVELS, ORD, J1, J2, J3, ALPHA, TESTPARAM, POWERPARAM)
%   estimates the power of the chi-square test of conditional independence 
%   performed by ORG.MENSXMACHINA.COMMONS.STATS.CHI2INDTESTOPT(X, NUMLEVELS, J1, J2, J3, PARAM)
%   with any PARAM as POWER = 1 if the mean observed count per cell (bin)
%   in the joint distribution of the data in the specified columns is
%   greater than or equal to POWERPARAM.Hps and POWER = 0 otherwise [1].
%   ALPHA, BETA and TESTPARAM are not used.
%
%   CHI2INDTESTHSPOPT is used as POWERFUN in constraint-based learning
%   functions in conjunction with 
%   ORG.MENSXMACHINA.PGM.INDTESTP.CHI2INDTESTPOPT as PFUN and
%   ORG.MENSXMACHINA.PGM.INDTESTPOWERMAXK.CHI2INDTESTHSPMAXKOPT as
%   POWERMAXKFUN.
%
%   [POWER POWERPPARAM] = CHI2INDTESTHSPOPT(X, NUMLEVELS, ORD, J1, J2, J3, ALPHA, BETA, TESTPARAM, POWERPARAM)
%   also returns an empty structure POWERPPARAM.
%
%   See also ORG.MENSXMACHINA.PGM.INDTESTP.CHI2INDTESTPOPT, 
%   ORG.MENSXMACHINA.PGM.INDTESTPOWERMAXK.CHI2INDTESTHSPMAXKOPT    

%   References:
%       [1] Aliferis, C. F., Statnikov, A., Tsamardinos, I., Mani, S., and
%       Koutsoukos, X. D. 2010. Local Causal and Markov Blanket Induction 
%       for Causal Discovery and Feature Selection for Classification Part
%       I: Algorithms and Empirical Evaluation. J. Mach. Learn. Res. 11
%       (Mar. 2010), 171-234.

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

import org.mensxmachina.commons.stats.*;

meanHps = size(x, 1) / prod(numLevels([j1 j2 j3]));

if meanHps >= Param.Hps
    power = 1;
else
    power = 0;
end

PowerPParam = struct();

end