function maxk = indtestpwrmaxkopt(x, numLevels, ~, j1, j2, j3, ~, ~, ~, PowerParam)
%INDTESTPWRMAXKOPT Maximal cardinality of a conditioning set with sufficient estimated power for generic test of conditional independence (optimized)
%   MAXK = INDTESTPWRMAXKOPT(X, NUMLEVELS, ORD, J1, J2, J3, ALPHA, BETA, TESTPARAM, POWERPARAM)
%   returns an upper bound MAXK on the maximal cardinality of a subset J4
%   of J3 such that 
%   POWER = ORG.MENSXMACHINA.PGM.INDTESTPOWER.INDTESTPWROPT(X, NUMLEVELS, ORD, J1, J2, J4, ALPHA, TESTPARAM, POWERPARAM)
%   returns a power estimate POWER >= 1 - BETA.
%
%   INDTESTPWRMAXKOPT is used as POWERMAXKFUN in constraint-based learning
%   functions in conjunction with 
%   ORG.MENSXMACHINA.PGM.INDTESTP.INDTESTPOPT as PFUN and 
%   ORG.MENSXMACHINA.PGM.INDTESTPOWER.INDTESTPWROPT as POWERFUN.
%
%   See also ORG.MENSXMACHINA.PGM.INDTESTP.INDTESTPOPT, 
%   ORG.MENSXMACHINA.PGM.INDTESTPOWER.INDTESTPWROPT

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

persistent Chi2IndTestHspParam;

if isempty(Chi2IndTestHspParam)
    Chi2IndTestHspParam = struct('Hps', NaN);
end

% update HPS
Chi2IndTestHspParam.Hps = PowerParam.Chi2IndTestHspParam.Hps;

if all(~isnan(numLevels))

    % all variables are categorical
    maxk = chi2indtesthspmaxkopt(x, numLevels, [], j1, j2, j3, [], [], [], Chi2IndTestHspParam);

else
    
    % some variables are numeric
    % return maximum possible conditioning set cardinality
    maxk = length(j3);

end

end