%% Identifying the skeleton of a Bayesian network
% This demonstration illustrates the skeleton identification process for
% the ALARM network.
%% Loading the true Bayesian network
% First we load the ALARM network as an object |bnet| of the
% |org.mensxmachina.pgm.CategoricalBayesianNetwork| class. 

% load ALARM network
load mxm_pgm_alarm_bnet; % loads 'bnet' Categorical Bayesian network

%% Loading the Bayesian network sample
% Then we load a sample from the ALARM network in dataset |a|.

% load ALARM network sample
load mxm_pgm_alarm_sample; % loads 'a' dataset

%% Learning the skeleton from the samples
% We provide the samples as input to the
% |org.mensxmachina.pgm.mmpcskeleton| function, which applies the MMPC
% algorithm to each of the variables in the dataset. We don't supply any 
% parameter-value pairs so the defaults are used. Note that by default no 
% symmetry correction is performed.

% learn the skeleton
skeleton = org.mensxmachina.pgm.mmpcskeleton(a);

%% Comparing the learned skeleton with the true one
% Now we compare the learned skeleton with the true ALARM skeleton. First
% we obtain the skeleton of the ALARM network by calling the |skeleton|
% method of the |org.mensxmachina.pgm.CategoricalBayesianNetwork| class. 
% Then we get a classification performance object with
% |org.mensxmachina.pgm.skeletonperf| and we print the sensitivity and 
% specificity. |org.mensxmachina.pgm.skeletonperf| is a version of the 
% |classperf| function in the Bioinformatics Toolbox (TM), specialized for 
% skeleton learning. A skeleton learning task can be viewed as a binary
% classification task where the possible edges are classified as edges 
% (positives) or non-edges (negatives). Finally we plot the confusion
% matrix with the |org.mensxmachina.pgm.plotskeletonconfusion| function,
% which is a version of the |plotconfusion| function in the Neural Network 
% Toolbox (TM), specialized for skeleton learning. 

% get true skeleton
skeleton_true = bnet.skeleton;

% get classifier performance object
cp = org.mensxmachina.pgm.skeletonperf(skeleton_true, skeleton);

% print sensitivity and specificity
fprintf('\nSensitivity = %.2f%%\n', cp.sensitivity*100);
fprintf('\nSpecificity = %.2f%%\n', cp.specificity*100);

% plot confusion matrix
org.mensxmachina.pgm.plotskeletonconfusion(skeleton_true, skeleton);