function [skeleton p stat Stats] = mmpcskeletonopt(x, numLevels, ord, Param)
%MMPCSKELETONOPT Optimized version of MMPCSKELETON
%   [...] = MMPCSKELETONOPT(X, NUMLEVELS, ORD, PARAM) is the optimized 
%   version of 
%   MMPCSKELETON(A, 'Param1', VAL1, 'Param2', VAL2, ..., 'ParamN', VALN).
%   X is a matrix of observations without any missing (NaN) values, as
%   returned by DOUBLE(A) after removing missing (undefined or NaN) values
%   from A with 
%   A(ANY(ORG.MENSXMACHINA.COMMONS.STATS.ISMISSING(A), 2), :) = [].
%   NUMLEVELS is a numeric vector of length equal to the number of
%   columns of X, containing their number of levels where the columns of X
%   contain categorical data in numeric form (i.e. value i corresponds to
%   the i-th level) and NaN where they contain truly numeric data.
%   NUMLEVELS is returned by ORG.MENSXMACHINA.COMMONS.STATS.NLEVELS(A).
%   ORD is a logical vector of length equal to the number of columns of X,
%   containing logical 1 (true) where the columns of X contain ordinal 
%   categorical data in numeric form and logical 0 (false) otherwise. ORD
%   is returned by DATASETFUN(@(IVAR) ISA(IVAR, 'ordinal'), A). PARAM is a
%   structure containing all MMPCSKELETON parameter name/value pairs 
%   'Param1', VAL1, 'Param2', VAL2, ..., 'ParamN', VALN as fields.
%
%   See also ORG.MENSXMACHINA.PGM.MMPCNONSYMOPT,
%   ORG.MENSXMACHINA.PGM.MMPCOPT, ORG.MENSXMACHINA.PGM.MMPCSKELETON

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

persistent InstParam;

if isempty(InstParam)
    InstParam = mmpcgllinstparam();
end

[skeleton p stat Stats] = lglskeletonopt(x, numLevels, ord, InstParam, Param);

end