function cp = pcperf(targetskeleton, outputskeleton, t)
%PCPERF Evaluate performance of parents and children learner
%
%   ORG.MENSXMACHINA.PGM.PCPERF is a version of the CLASSPERF function in
%   the Bioinformatics Toolbox (TM), specialized for parents and children
%   learning. For a given node, the task of learning its parents and
%   children can be viewed as a binary classification task, where each
%   other node is classified as either parent or child, or not.
%
%   CP = PCPERF(TARGETSKELETON, OUTPUTPC, T) returns a classifier
%   performance object CP for the output parents and children set OUTPUTPC,
%   returned either by ORG.MENSXMACHINA.PGM.MMPC or
%   ORG.MENSXMACHINA.PGM.MMPCBAR, targeting the parents and children of
%   the variable specified by positive integer T in skeleton
%   TARGETSKELETON.
%
%   Examples:
%
%       % load ALARM network
%       load mxm_bnet_alarm_net; % loads 'net' BayesianNetwork
%
%       % load ALARM network samples
%       load mxm_bnet_alarm_sample; % loads 'a' dataset
%
%       % learn parents and children set of variable #37
%       pc = org.mensxmachina.pgm.mmpc(a, 37);
%
%       % get true skeleton
%       skeleton_true = net.skeleton;
%
%       % get classifier performance object for variable #37
%       cp = org.mensxmachina.pgm.pcperf(skeleton_true, pc, 37);
%
%   See also: CLASSPERF, ORG.MENSXMACHINA.PGM.SKELETONPERF

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

targets = targetskeleton(t, :);
targets(t) = []; % remove target variable

outputs = outputskeleton(t, :);
outputs(t) = []; % remove target variable

% get parents and children learning performance
cp = classperf(double(targets), double(outputs), 'Positive', 1, 'Negative', 0); 

end