function plotskeletonconfusion(targetskeleton, outputskeleton)
%PLOTSKELETONCONFUSION Plot skeleton confusion matrix 
%   ORG.MENSXMACHINA.PGM.PLOTSKELETONCONFUSION is a version of the
%   PLOTCONFUSION function in the Neural Network Toolbox (TM), specialized
%   for skeleton learning. A skeleton learning task can be viewed as a 
%   binary classification task, where the possible edges are classified as
%   edges (positives) or non-edges (negatives). 
%   
%   PLOTSKELETONCONFUSION(TARGETSKELETON, OUTPUTSKELETON) displays the 
%   classification confusion grid of a skeleton learning outcome. 
%   TARGETSKELETON is the target skeleton and OUTPUTSKELETON is the
%   output of the skeleton learner.
%
%   See also: PLOTCONFUSION

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

% convert true skeleton to column vector
skeletonvector_true = org.mensxmachina.commons.graph.skeleton2vector(targetskeleton);

% convert skeleton to column vector
skeletonvector = org.mensxmachina.commons.graph.skeleton2vector(outputskeleton);

plotconfusion(skeletonvector_true', skeletonvector'); 

end