function cpdfParam = categoricalcpdfparam( Param, parentVal )
%CATEGORICALCPDFPARAM Categorical Bayesian Network node conditional pdf parameters given parent values
%   CPDFPARAM = CATEGORICALCPDFPARAM(PARAM, PARENTVAL) returns the
%   conditional probability distribution function parameters CPDFPARAM of a
%   Categorical Bayesian Network node with parameters PARAM, given the 
%   values of its parents in PARENTVAL.

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

% get # of samples
n = size(parentVal, 1);

% get the conditional pdf parameters of the node for each sample
parentVal = num2cell( parentVal );
cpdfParam = {cell2mat(arrayfun(@(i) Param( :, parentVal{i, :} )', (1:n)', 'UniformOutput', false))};

end