function [x numLevels ord Param] = gllparamparse(functionName, varargin)

import org.mensxmachina.pgm.*;
import org.mensxmachina.pgm.indtestp.*;
import org.mensxmachina.pgm.indtestpower.*;
import org.mensxmachina.pgm.indtestpowermaxk.*;
import org.mensxmachina.commons.*;
import org.mensxmachina.commons.stats.*;

ip = inputParser; % Create an instance of the inputParser class
ip.FunctionName = functionName;

if ~isa(varargin{1}, 'dataset')
    
    ip.addRequired('x', @(x) isnumeric(x) && ndims(x) == 2);
    ip.addRequired('numLevels', @(numLevels) isnumeric(numLevels) && isvector(numLevels));
    ip.addRequired('ord', @(ord) islogical(ord) && isvector(ord));
    
end

ip.addParamValue('Alpha', 0.05, @(alpha) isscalar(alpha) && alpha >= 0 && alpha <= 1);
ip.addParamValue('Beta', 0, @(beta) isscalar(beta) && beta >= 0 && beta <= 1);
%ip.addParamValue('ControlledErrorRate', 'fpr', @(controlledErrorRate) ischar(controlledErrorRate) && ismember(controlledErrorRate, {'fpr', 'fdr', 'pfdr'}));
%ip.addParamValue('Bycm', false, @(bycm) isscalar(bycm) && islogical(bycm) );
%ip.addParamValue('Lambda', 0, @(lambda) isnumeric(lambda) && isreal(lambda) && isscalar(lambda) && lambda >= 0 && lambda <=1 );
ip.addParamValue('MaxK', 10, @(maxK) isnonnegativeinteger(maxK) || isinf(maxK));
ip.addParamValue('PowerFun', [], @(powerFun) isa(powerFun, 'function_handle'));
ip.addParamValue('PowerMaxKFun', [], @(powerMaxKFun) isa(powerMaxKFun, 'function_handle'));
ip.addParamValue('PowerParam', [], @isstruct);
%ip.addParamValue('Q', 0.05, @(q) isscalar(q) && q >= 0 && q <= 1);
ip.addParamValue('PFun', [], @(testFun) isa(testFun, 'function_handle'));
ip.addParamValue('PParam', [], @isstruct);
ip.addParamValue('TestParam', [], @isstruct);
ip.addParamValue('Verbose', false, @islogical);

if isa(varargin{1}, 'dataset')
    
    ip.parse(varargin{2:end});
    
    Param = ip.Results;
    
    % remove rows with missing (undefined or NaN) values
    varargin{1}(any(ismissing(varargin{1}), 2), :) = [];

    % convert to double
    x = double(varargin{1});

    % get # of levels vector
    numLevels = nlevels(varargin{1});

    % find ordinal variables
    ord = datasetfun(@(var) isa(var, 'ordinal'), varargin{1});
    
else
    
    ip.parse(varargin{:});
    
    Param = ip.Results;
    
    % check consistency
    
    assert(length(Param.numLevels) == size(Param.x, 2), ...
        ['org:mensxmachina:pgm:' functionName ':NumLevelsLengthNotNumColsInX'], ...
        'The length of NUMLEVELS is not equal to the number of columns of X.');
    
    assert(length(Param.ord) == size(Param.x, 2), ...
        ['org:mensxmachina:pgm:' functionName ':OrdLengthNotNumColsInX'], ...
        'The length of ORD is not equal to the number of columns of X.');
    
    x = Param.x;
    numLevels = Param.numLevels;
    ord = Param.ord;
    
    % remove rows with NaN values
    x(any(isnan(x), 2), :) = [];
    
end

% convert CONTROLLEDERRORRATE to integer
%Param.ControlledErrorRate = find(strcmp(Param.ControlledErrorRate, {'fpr', 'fdr', 'pfdr'}), 1);

% assign default values to parameters if they are empty

% find which of the variables are numeric
isNumericVar = isnan(numLevels);

allVarsAreCategorical = all(~isNumericVar);
allVarsAreNumeric = all(isNumericVar);

if isempty(Param.PowerFun)      
    if allVarsAreCategorical % all variables are categorical
        Param.PowerFun = @chi2indtesthspopt;
    elseif allVarsAreNumeric % all variables are numeric
        Param.PowerFun = @dummyindtestpwropt;
    else % mixed variables
        Param.PowerFun = @indtestpwropt;
    end
end

if isempty(Param.PowerMaxKFun)      
    if allVarsAreCategorical % all variables are categorical
        Param.PowerMaxKFun = @chi2indtesthspmaxkopt;
    elseif allVarsAreNumeric % all variables are numeric
        Param.PowerMaxKFun = @dummyindtestpwrmaxkopt;
    else % mixed variables
        Param.PowerMaxKFun = @indtestpwrmaxkopt;
    end
end

if isempty(Param.PowerParam)   
    if allVarsAreCategorical % all variables are categorical
        Param.PowerParam = struct('Hps', 5);
    elseif allVarsAreNumeric % all variables are numeric
        Param.PowerParam = [];
    else % mixed variables
        Param.PowerParam = struct('Chi2IndTestHspParam', struct('Hps', 5));
    end   
end

if isempty(Param.PFun)   
    if allVarsAreCategorical % all variables are categorical
        Param.PFun = @chi2indtestpopt;
    elseif allVarsAreNumeric % all variables are numeric
        Param.PFun = @fisherzindtestpopt;
    else % mixed variables
        Param.PFun = @indtestopt;
    end   
end

if isempty(Param.TestParam)
    if allVarsAreCategorical % all variables are categorical
        Param.TestParam = struct('Stat', 2);
    elseif allVarsAreNumeric % all variables are numeric
        Param.TestParam = struct();
    else % mixed variables
        Param.TestParam = struct('Chi2IndTestParam', struct('Stat', 2));
    end   
end

end