function [skeleton p stat flag Stats] = gllpcnonsymopt(x, numLevels, ord, j, skeleton, p, stat, flag, Stats, InstParam, Param)
%GLLPCNONSYMOPT GLL-PC-nonsym

%   References:
%       [1] Aliferis, C. F., Statnikov, A., Tsamardinos, I., Mani, S., and
%       Koutsoukos, X. D. 2010. Local Causal and Markov Blanket Induction 
%       for Causal Discovery and Feature Selection for Classification Part
%       I: Algorithms and Empirical Evaluation. J. Mach. Learn. Res. 11
%       (Mar. 2010), 171-234.

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

% params: *Fun, *Param, verbose

% create TPC(J), the set of tentative parents and children of J

% initialize TPC(J), the set of tentative parents and children of J
[skeleton p stat tpc Stats] = InstParam.TpcInitFun(x, numLevels, ord, j, skeleton, p, stat, flag, Stats, InstParam.TpcInitParam, Param);

% initialize OPEN to V\({J} U TPC(J))

% initialize OPEN to V
open = 1:size(skeleton, 1);

% remove J and TPC(J) from OPEN
open = setdiff(open, [j tpc]);

% init OPEN priority queue
[openQueue Stats] = InstParam.OpenQueueInitFun(x, numLevels, ord, j, skeleton, p, stat, flag, tpc, open, Stats, InstParam.OpenQueueInitParam, Param);

while true % apply interleaving strategy...

    % apply inclusion heuristic function

    % a) prioritize variables in OPEN for inclusion in TPC(J) and
    % b) throw away non-eligible variables from OPEN
    
    [skeleton p stat open openQueue Stats] = ...
        InstParam.OpenUpdateFun(x, numLevels, ord, j, skeleton, p, stat, flag, tpc, open, openQueue, Stats, InstParam.OpenUpdateParam, Param);

    %{                      
    if Param.Verbose
        fprintf('\n[%s] thrown from OPEN.\n', num2str(thrown));
    end
    %}
        
    % c) insert in TPC(J) the highest priority variable(s) in OPEN and
    % remove them from OPEN
    
    [tpc open openQueue Stats] = ...
        InstParam.OpenRmFun(x, numLevels, ord, j, skeleton, p, stat, flag, tpc, open, openQueue, Stats, InstParam.OpenRmParam, Param);
    
    if Param.Verbose
        %fprintf('\n[%s] entered TPC(%d).\n', num2str(removed), j);
        fprintf('\nNew TPC(%d) = [%s].\n', j, num2str(tpc));
    end
    
    % apply elimination strategy to remove variables from TPC(J)
    [skeleton p stat tpc Stats] = ...
        InstParam.TpcRmFun(x, numLevels, ord, j, skeleton, p, stat, flag, tpc, open, Stats, InstParam.TpcRmParam, Param);
    
    if Param.Verbose
        %fprintf('\n[%s] removed from TPC(%d).\n', num2str(eliminated), j);
        fprintf('\nNew TPC(%d) = [%s].\n', j, num2str(tpc));
    end
    
    [terminate Stats] = ...
        InstParam.TerminateFun(x, numLevels, ord, j, skeleton, p, stat, flag, tpc, open, Stats, InstParam.TerminateParam, Param);
    
    if terminate
        % ... until the termination criterion is met
        break;        
    end

end

flag(j) = true;

if Param.Verbose
    fprintf('\nFinal TPC(%d) = [ %s]\n', j, num2str(tpc));
end

end