function [skeleton p stat Stats] = gllskeletonupdateopt(x, numLevels, ord, j2, j, j3, skeleton, p, stat, flag, Stats, Param)
%GLLSKELETONUPDATEOPT Update skeleton after considering I(j2,j|j3)

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

if flag(j2) && ~skeleton(j, j2)
    return;
end

if Param.Verbose
    fprintf('\nDetermining I(%d; %d | [ %s])...', j2, j, num2str(j3));
end

% update # of tests considered
Stats.numTestsConsidered(j, j2) = Stats.numTestsConsidered(j, j2) + 1;
Stats.numTestsConsidered(j2, j) = Stats.numTestsConsidered(j2, j) + 1;

% calculate test power
[power PowerPParam] = Param.PowerFun(x, numLevels, ord, j2, j, j3, Param.Alpha, Param.TestParam, Param.PowerParam);

if 1 - power <= Param.Beta % there is sufficient power

    % calculate test p-value
    [p_j2_dsep_j_j3 stat_j2_dsep_j_j3] = Param.PFun(x, numLevels, ord, j2, j, j3, Param.TestParam, PowerPParam, Param.PParam);

    % update # of tests attempted
    Stats.numTestsAttempted(j, j2) = Stats.numTestsAttempted(j, j2) + 1;
    Stats.numTestsAttempted(j2, j) = Stats.numTestsAttempted(j2, j) + 1;
    
    if ~isnan(p_j2_dsep_j_j3) % if p-value was successfully computed (not NaN) 
        
        % set the edge
        h = p_j2_dsep_j_j3 <= Param.Alpha;
        skeleton(j, j2) = h;
        skeleton(j2, j) = h;
        
        % update # of tests
        Stats.numTestsSucceeded(j, j2) = Stats.numTestsSucceeded(j, j2) + 1;
        Stats.numTestsSucceeded(j2, j) = Stats.numTestsSucceeded(j2, j) + 1;

        if isempty(j3) || pcmp(p_j2_dsep_j_j3, stat_j2_dsep_j_j3, p(j, j2), stat(j, j2)) > 0

            % update edge p-value and statistic
            
            p(j, j2) = p_j2_dsep_j_j3;
            p(j2, j) = p_j2_dsep_j_j3;

            stat(j, j2) = stat_j2_dsep_j_j3;
            stat(j2, j) = stat_j2_dsep_j_j3;
            
        end

        %if ~Param.IncludeNonSkeletonStats && ~skeleton(j, j2)
        if ~skeleton(j, j2)
            
            % set stats to 0
            p(j, j2) = 0;
            p(j2, j) = 0;
            
            stat(j, j2) = 0;
            stat(j2, j) = 0;
            
            Stats.numTestsConsidered(j, j2) = 0;
            Stats.numTestsConsidered(j2, j) = 0;
            
            Stats.numTestsAttempted(j, j2) = 0;
            Stats.numTestsAttempted(j2, j) = 0;
            
            Stats.numTestsSucceeded(j, j2) = 0;
            Stats.numTestsSucceeded(j2, j) = 0;

        end
        
        if Param.Verbose
            if ~skeleton(j, j2)
                fprintf(' Yes (%f. %f).\n', p_j2_dsep_j_j3, stat_j2_dsep_j_j3);
            else
                fprintf(' No (%f, %f).\n', p_j2_dsep_j_j3, stat_j2_dsep_j_j3);
            end
        end

    end % else ignore the test    
    
end % else ignore the test

end