function [pfdr q] = lglskeletonpermpfdropt(skeleton, p, Param)

import org.mensxmachina.commons.*;
import org.mensxmachina.commons.stats.*;

% convert skeleton matrix to vector
skeleton_vector = org.mensxmachina.commons.graph.skeleton2vector(skeleton);
skeleton_vector_ind = find(skeleton_vector);

% convert maximum p-value matrix to vector
p_vector = org.mensxmachina.commons.graph.skeleton2vector(p);

% keep only skeleton maximum p-values and convert to full
p_skeleton_vector = full(p_vector(skeleton_vector_ind));
    
% get the size of the dataset
[n m] = size(Param.Dataset);

% sort skeleton maximum p-value vector
[p_skeleton_vector_sorted p_skeleton_vector_order] = sort(p_skeleton_vector);

p_skeleton_vector_sorted = [ -p_skeleton_vector_sorted(end:-1:1); Inf];

r0_p_skeleton_vector = zeros(size(p_skeleton_vector));
er0_p_skeleton_vector = zeros(size(p_skeleton_vector));

for i=1:Param.B % for each permutation

    fprintf('\nRunning permutation #%d...\n', i);
    
    % initialize p
    p_perm = zeros(m, m);
    
    for j=1:m % for each node J
    
        fprintf('\nSimulating null maximum p-values for edges with end node #%d...\n', j);
        
        % permute J values
        % more rigorously, we should do this for each edge and not for each
        % node, but that would take too much time...
        dataset_perm = Param.Dataset;
        dataset_perm(:, j) = dataset_perm(randperm(n), j);
    
        % call GLL-PC-nonsym with target J
        [~, p_j] = ...
            Param.GllPcNonSymFun(dataset_perm, m, j, true, [], Param.GllPcNonSymParam);
    
        for j2=1:m % for each other node J2
            
            if j2 > j
                % use only lower triangular
                break;
            end
            
            % call GLL-PC-nonsym with target J2
            [~, p_j2] = ...
                Param.GllPcNonSymFun(dataset_perm, m, j2, true, [], Param.GllPcNonSymParam);
        
            p_perm(j, j2) = max(p_j(j2), p_j2(j));
            
        end
        
    end

    % convert maximum p-value matrix to vector; note that 
    % ORG.MENXMACHINA.COMMONS.GRAPH.SKELETON2VECTOR uses only the lower 
    % triangular part of the matrix, i.e. the one we have filled
    p_perm_vector = org.mensxmachina.commons.graph.skeleton2vector(p_perm);

    % -p_(k) <= -p < -p_(k-1) <=> p_(k-1) < p <= p_(k)
    counts = histc(-p_perm_vector, p_skeleton_vector_sorted);

    r0_p_skeleton_vector(p_skeleton_vector_order) = cumsum(counts(end-1:-1:1));

    er0_p_skeleton_vector = er0_p_skeleton_vector + r0_p_skeleton_vector;
    
end

er0_p_skeleton_vector = er0_p_skeleton_vector/Param.B;

if nargout == 1
    
    % avoid calculation of q-values by ORG.MENSXMACHINA.COMMONS.STATS.FDR
    % if they are not returned
    pfdr = org.mensxmachina.commons.stats.fdr(p_skeleton_vector, 'ER0p', er0_p_skeleton_vector);
    
else
    
    [pfdr q] = org.mensxmachina.commons.stats.fdr(p_skeleton_vector, 'ER0p', er0_p_skeleton_vector);
    
    % create skeleton q-value vector
    q = sparse(skeleton_vector_ind, 1, q, 1, length(skeleton_vector), 1);
    
    % convert q-value vector to matrix
    q = org.mensxmachina.commons.graph.vector2skeleton(q);
    
end

% create skeleton FDR vector
pfdr = sparse(skeleton_vector_ind, 1, pfdr, length(skeleton_vector), 1);
    
% convert FDR vector to matrix
pfdr = org.mensxmachina.commons.graph.vector2skeleton(pfdr);

end