function [skeleton p stat open openQueue Stats] = mmpcopenupdateopt(x, numLevels, ord, j, skeleton, p, stat, flag, tpc, open, openQueue, Stats, ~, Param)
%MMPCOPENUPDATEOPT Update MMPC OPEN

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

% calculates associations of the remaining variables in OPEN with J 
% given the new subsets of TPC(J), while returning any variables
% that achieve no association

for j2IndInOpen = 1:numel(open) % for each variable in OPEN

    % let it be J2
    j2 = open(j2IndInOpen);
    
    % find the maximal conditioning set cardinality with sufficient power
    powerMaxK = Param.PowerMaxKFun(x, numLevels, ord, j2, j, tpc, Param.Alpha, Param.Beta, Param.TestParam, Param.PowerParam);

    % get minimum between sufficient power maxk and user MAXK
    maxK = min(powerMaxK, Param.MaxK);
        
    if isempty(tpc)

        % test I(J2;J|[])
        [skeleton p stat Stats] = gllskeletonupdateopt(x, numLevels, ord, j2, j, [], skeleton, p, stat, flag, Stats, Param);
        
        if isnan(openQueue.assocMin(j2IndInOpen)) || assoccmp(-p(j, j2), stat(j, j2), openQueue.assocMin(j2IndInOpen), openQueue.assocMinStat(j2IndInOpen)) > 0
            
            % update minimum association and corresponding statistic
            openQueue.assocMin(j2IndInOpen) = -p(j, j2);
            openQueue.assocMinStat(j2IndInOpen) = stat(j, j2);
        
        end
        
        continue;

    end

    if Param.Verbose
        fprintf('\nmax |J3| for a subset J3 of TPC(%d) = [%s] in testing I(%d;%d|J3): %d of %d\n', j, num2str(tpc), j2, j, maxK, numel(tpc));
    end

    if maxK == 0 
        
        % only I(J2;J|[]) can be tested accuratelly
        % and it's already tested
        continue;
        
    end % else maxK > 0

    % test I(J2;J|F)
    f = tpc(end);
    [skeleton p stat Stats] = gllskeletonupdateopt(x, numLevels, ord, j2, j, f, skeleton, p, stat, flag, Stats, Param);            
        
    if isnan(openQueue.assocMin(j2IndInOpen)) || assoccmp(-p(j, j2), stat(j, j2), openQueue.assocMin(j2IndInOpen), openQueue.assocMinStat(j2IndInOpen)) > 0
            
        % update minimum association and corresponding statistic
        openQueue.assocMin(j2IndInOpen) = -p(j, j2);
        openQueue.assocMinStat(j2IndInOpen) = stat(j, j2);

    end
        
    if ~skeleton(j, j2)
        continue; % no need to try more subsets
    end

    if maxK < 2
        % |J3| == 0 ([]) and |J3| == 1 (F) already tested,
        % no other |J3| < 2 to test
        continue;                
    end

    if numel(tpc(end)) == 2 % if |TPC(J)| == 2
    
        % test I(J2;J|TPC(J))
        [skeleton p stat Stats] = gllskeletonupdateopt(x, numLevels, ord, j2, j3, tpc, skeleton, p, stat, flag, Stats, Param);
        
        if isnan(openQueue.assocMin(j2IndInOpen)) || assoccmp(-p(j, j2), stat(j, j2), openQueue.assocMin(j2IndInOpen), openQueue.assocMinStat(j2IndInOpen)) > 0

            % update minimum association and corresponding statistic
            openQueue.assocMin(j2IndInOpen) = -p(j, j2);
            openQueue.assocMinStat(j2IndInOpen) = stat(j, j2);

        end
    
        % we tested with all subsets of both variables,
        % no more subsets to try
        continue;
    
    end % else |TPC(J)| > 2

    for nj3 = 2:maxK % for each cardinality up to the maximum

        % find subsets of this size - 1 among the previous TPC(J)
        C = nchoosek( tpc(1:end-1), nj3 - 1); 

        for k = 1:size(C, 1) % for each such subset

            % add F and let it be J3 
            j3 = [ C(k, :) tpc(end)];
            
            % i.e. for each subset J3 of TPC(J) including F
            % and with cardinality up to the maximum

            % test I(J2;J|J3)
            [skeleton p stat Stats] = gllskeletonupdateopt(x, numLevels, ord, j2, j, j3, skeleton, p, stat, flag, Stats, Param);                
        
            % update
            
            if isnan(openQueue.assocMin(j2IndInOpen)) || assoccmp(-p(j, j2), stat(j, j2), openQueue.assocMin(j2IndInOpen), openQueue.assocMinStat(j2IndInOpen)) > 0

                % update minimum association and corresponding statistic
                openQueue.assocMin(j2IndInOpen) = -p(j, j2);
                openQueue.assocMinStat(j2IndInOpen) = stat(j, j2);

            end
        
            if ~skeleton(j, j2)
                break;
            end

        end

        if ~skeleton(j, j2)
            break;
        end
        
    end            
    
end

% throw variables with no association with J given TPC(J) from OPEN

noAssocOpen = ~skeleton(j, open);

open(noAssocOpen) = [];

openQueue.assocMin(noAssocOpen) = [];
openQueue.assocMinStat(noAssocOpen) = [];

end