classdef TestMmpc < TestCase
%TESTMMPC MMPC test cases

methods

function obj = TestMmpc(name)

obj = obj@TestCase(name);

end

function testNonSym(obj)
    
import org.mensxmachina.pgm.*;
    
% load ALARM sample #1
load('mxm_pgm_alarm_sample', 'a');

% keep 100 samples
a = a(1:100, :);

% learn PC(26)

%[skeleton p stat flag Stats] = mmpcnonsym(a, 26);
%save('mmpcnonsym_result_default', 'skeleton', 'p', 'stat', 'flag', 'Stats');

[skeleton1 p1 stat1 flag1 Stats1] = mmpcnonsym(a, 26);
load('mmpcnonsym_result_default', 'skeleton', 'p', 'stat', 'flag', 'Stats');

% compare with the correct output
assertEqual(skeleton, skeleton1);
assertEqual(p, p1);
assertEqual(stat, stat1);
assertEqual(flag, flag1);
assertEqual(Stats, Stats1);

end

function testNonSymMatrix(obj)
    
import org.mensxmachina.pgm.*;
import org.mensxmachina.commons.stats.*; 
    
% load ALARM sample #1
load('mxm_pgm_alarm_sample', 'a');

% keep 100 samples
a = a(1:100, :);

% learn PC(26)

% convert to double
x = double(a);
 
% get # of levels vector
numLevels = nlevels(a);
 
% find ordinal variables
ord = datasetfun(@(var) isa(var, 'ordinal'), a);

[skeleton1 p1 stat1 flag1 Stats1] = mmpcnonsym(x, numLevels, ord, 26);
load('mmpcnonsym_result_default', 'skeleton', 'p', 'stat', 'flag', 'Stats');

% compare with the correct output
assertEqual(skeleton, skeleton1);
assertEqual(p, p1);
assertEqual(stat, stat1);
assertEqual(flag, flag1);
assertEqual(Stats, Stats1);

end

function testSym(obj)
% test Sym param
    
import org.mensxmachina.pgm.*;    

% load ALARM sample #1
load('mxm_pgm_alarm_sample', 'a');

% keep 100 samples
a = a(1:100, :);

% learn PC(26)
%[skeleton p stat flag Stats] = mmpc(a, 26);
%save('mmpc_result_default', 'skeleton', 'p', 'stat', 'flag', 'Stats');

[skeleton1 p1 stat1 flag1 Stats1] = mmpc(a, 26);
load('mmpc_result_default', 'skeleton', 'p', 'stat', 'flag', 'Stats');

% compare with the correct output
assertEqual(skeleton, skeleton1);
assertEqual(p, p1);
assertEqual(stat, stat1);
assertEqual(flag, flag1);
assertEqual(Stats, Stats1);

end

function testSymMatrix(obj)
% test Sym param
    
import org.mensxmachina.pgm.*;    
import org.mensxmachina.commons.stats.*;  

% load ALARM sample #1
load('mxm_pgm_alarm_sample', 'a');

% keep 100 samples
a = a(1:100, :);

% convert to double
x = double(a);
 
% get # of levels vector
numLevels = nlevels(a);
 
% find ordinal variables
ord = datasetfun(@(var) isa(var, 'ordinal'), a);

[skeleton1 p1 stat1 flag1 Stats1] = mmpc(x, numLevels, ord, 26);
load('mmpc_result_default', 'skeleton', 'p', 'stat', 'flag', 'Stats');

% compare with the correct output
assertEqual(skeleton, skeleton1);
assertEqual(p, p1);
assertEqual(stat, stat1);
assertEqual(flag, flag1);
assertEqual(Stats, Stats1);

end

function testP(obj)
    
import org.mensxmachina.pgm.*;
import org.mensxmachina.commons.*;
import org.mensxmachina.commons.stats.*;
    
% load ALARM sample #1
load('mxm_pgm_alarm_sample', 'a');

% keep 100 samples
a = a(1:100, :);

% learn PC(26)
[skeleton p] = mmpc(a, 26);
    
% assert all PC have p-values <= Alpha
assertTrue( all(p(26, ~skeleton(26, :)) <= 0.05) );

% learn PC(26)
%[skeleton p stat flag Stats] = mmpc(a, 26);

% assert all NPC have
% either max p-values > Alpha or NaN max p-values
%assertTrue( all( p(26, ~skeleton(26, :)) > 0.05 | isnan(p(26, ~skeleton(26, :)) ) ) );

end

end

end