classdef TestMmpcSkeleton < TestCase
%TESTMMPCSKELETON mmpcskeleton test cases

properties
    
    %{
    bnetParam % bnet parameters
    sampleParam % sampling parameters
    learnParam % learning parameters
    
    config_default
    %}
    
end

methods

function obj = TestMmpcSkeleton(name)

obj = obj@TestCase(name);

end
%{
function testIncludeNonSkeletonStats(obj)
% test IncludeNonSkeletonStats parameter

import org.mensxmachina.pgm.*;
import org.mensxmachina.commons.*;
import org.mensxmachina.commons.stats.*;

load('mxm_pgm_alarm_sample', 'a'); % load sample

SkeletonUpdateParam = struct('Alpha', 0.05, 'Beta', 0, 'PFun', @chi2indtestopt, 'TestParam', struct('Stat', 2), 'PowerFun', @chi2indtesthspopt, 'PowerParam', struct('Hps', 5), 'IncludeNonSkeletonStats', true);

[skeleton1 Stats1] = mmpcskeleton(a, 'SkeletonUpdateParam', SkeletonUpdateParam);
[skeleton2 Stats2] = mmpcskeleton(a);

assertEqual( skeleton1, skeleton2 );

isSkeleton = find(skeleton1);

assertTrue( issparse(skeleton1) );
assertTrue( ~issparse(Stats1.p) );
assertTrue( ~issparse(Stats1.stat) );

assertTrue( issparse(skeleton2) );
assertTrue( issparse(Stats2.p) );
assertTrue( issparse(Stats2.stat) );

assertEqual( Stats1.p(isSkeleton), full(Stats2.p(isSkeleton)) );
assertEqual( Stats1.stat(isSkeleton), full(Stats2.stat(isSkeleton)) );

assertTrue( ~any(full(Stats2.p(~isSkeleton))) );
assertTrue( ~any(full(Stats2.stat(~isSkeleton))) );

end
%}

function testVerbose(obj)

import org.mensxmachina.pgm.*;

load('mxm_pgm_alarm_sample', 'a'); % load sample
 
a = a(1:100, :); % select first 100 samples

[skeleton p stat Stats] = mmpcskeleton(a, 'Verbose', true);
[skeleton1 p1 stat1 Stats1] = mmpcskeleton(a);

assertEqual(skeleton, skeleton1);
assertEqual(p, p1);
assertEqual(stat, stat1);
assertEqual(Stats, Stats1);

end

function testDefault(obj)
    
import org.mensxmachina.pgm.*;

load('mxm_pgm_alarm_sample', 'a'); % load sample
 
a = a(1:500, :); % select first 500 samples


%[skeleton p stat Stats] = mmpcskeleton(a);
%save('mmpcskeleton_result_default', 'skeleton', 'p', 'stat', 'Stats');

[skeleton1 p1 stat1 Stats1] = mmpcskeleton(a);
load('mmpcskeleton_result_default', 'skeleton', 'p', 'stat', 'Stats');

assertEqual(skeleton, skeleton1);
assertEqual(p, p1);
assertEqual(stat, stat1);
assertEqual(Stats, Stats1);

load('mxm_pgm_alarm_bnet', 'bnet');
skeleton_true = bnet.skeleton;

% get performance object
slp = skeletonperf(skeleton_true, skeleton);

realizedPower = slp.sensitivity
realizedFpr = 1 - slp.specificity
realizedFdr = skeletonrlzdfdr(skeleton, p, skeleton_true);

[row col] = skeletonstepup(skeleton, p, p, 0.05);

realizedFdr = realizedFdr(row, col)

%{
% convert skeleton to vector
skeleton_vector = org.mensxmachina.commons.graph.skeleton2vector(skeleton1);

% convert p-value matrix to vector
p_vector = org.mensxmachina.commons.graph.skeleton2vector(p1);

% convert true skeleton to vector
skeleton_true_vector = org.mensxmachina.commons.graph.skeleton2vector(skeleton_true);

% keep false discovery p-values only and convert to full
fdp = full(p_vector(skeleton_vector & ~skeleton_true_vector))

hist(fdp);
%}

end

function testDefaultMatrix(obj)
    
import org.mensxmachina.pgm.*;
import org.mensxmachina.commons.stats.*;

load('mxm_pgm_alarm_sample', 'a'); % load sample
 
a = a(1:500, :); % select first 500 samples

%[skeleton p stat Stats] = mmpcskeleton(a);
%save('mmpcskeleton_result_default', 'skeleton', 'p', 'stat', 'Stats');

% convert to double
x = double(a);

% get # of levels vector
numLevels = nlevels(a);

% find ordinal variables
ord = datasetfun(@(var) isa(var, 'ordinal'), a);

[skeleton1 p1 stat1 Stats1] = mmpcskeleton(x, numLevels, ord);
load('mmpcskeleton_result_default', 'skeleton', 'p', 'stat', 'Stats');

assertEqual(skeleton, skeleton1);
assertEqual(p, p1);
assertEqual(stat, stat1);
assertEqual(Stats, Stats1);

end

%{
function testPower(obj)
    
import org.mensxmachina.pgm.*;

load('mxm_pgm_alarm_sample', 'a'); % load sample
 
%a = a(1:500, :); % select first 500 samples

load('mxm_pgm_alarm_bnet', 'bnet');

skeleton_true = bnet.skeleton;

Param = { ...
    'Beta', 0.05, ...
    'PowerFun', @chi2indtestpwropt, ...
    'PowerMaxKFun', @dummyindtestpwrmaxkopt, ...
    'PowerParam', struct('W', 0.0778) ...
    };

skeleton1 = mmpcskeleton(a);
skeleton2 = mmpcskeleton(a, Param{:});

slp = skeletonperf(skeleton_true, skeleton1);
realizedPower = slp.sensitivity
realizedFpr = 1 - slp.specificity

slp = skeletonperf(skeleton_true, skeleton2);
realizedPower = slp.sensitivity
realizedFpr = 1 - slp.specificity
    
end
%}
function testCheese(obj)
   
a = dataset('File', 'cheese.txt', 'ReadObsNames', false);

% learn the skeleton
skeleton = org.mensxmachina.pgm.mmpcskeleton(a);

% view it
%view(biograph(tril(skeleton), get(a, 'VarNames'), 'ShowArrows', 'off')); 

end

function testSmokingAndCancer(obj)
   
a = dataset('File', 'smokingandcancer.txt', 'ReadObsNames', false);

% learn the skeleton
skeleton = org.mensxmachina.pgm.mmpcskeleton(a);

% view it
%view(biograph(tril(skeleton), get(a, 'VarNames'), 'ShowArrows', 'off'));        
    
end

end

end