classdef TestXdslRead < TestCase
%TESTBAYESIANNETWORK BayesianNetwork class test cases

properties
    bnet
    node
end

methods

function obj = TestXdslRead(name)

obj = obj@TestCase(name);

end

function testcmpwithbif(obj)

import org.mensxmachina.pgm.*;

bnet_bif = bifread('alarm.bif', 'nominal');
bnet_xdsl = xdslread('alarm.xdsl', 'nominal');

[~, varOrder_bif] = sort(bnet_bif.VarNames);

varOrder_xdsl = [10 25 37 26 36 5 15 29 8 32 12 2 27 28 30 31 3 9 13 14 4 1 33 16 34 6 35 22 21 24 23 7 11 20 19 17 18];

G_ordered_bif = bnet_bif.G(varOrder_bif, varOrder_bif);
G_ordered_xdsl = bnet_xdsl.G(varOrder_xdsl, varOrder_xdsl);

assertEqual(G_ordered_bif, G_ordered_xdsl);
assertEqual(bnet_bif.NumLevels(varOrder_bif), bnet_xdsl.NumLevels(varOrder_xdsl));

Param_bif = bnet_bif.Param(varOrder_bif);
Param_xdsl = bnet_xdsl.Param(varOrder_xdsl);

% the levels of these variables are known to be permuted among the bif and
% the xsdl files; ignore them when checking if CPTs are equal
permutedLevelsVarNames = ...
    {'CATECHOL', 'EXPCO2', 'HRBP', 'HREKG', 'HRSAT', 'LVEDVOLUME', 'MINVOL', 'PRESS', 'PVSAT', 'SAO2', 'STROKEVOLUME', 'VENTALV', 'VENTLUNG', 'VENTTUBE'};

for i=1:bnet_bif.NumNodes
    
    i_bif = varOrder_bif(i);
    i_xdsl = varOrder_xdsl(i);
    
    varName = bnet_bif.VarNames{i_bif};
    
    if ismember(varName, permutedLevelsVarNames)
        continue;
    end
    
    numParents = length(org.mensxmachina.commons.graph.parents(bnet_bif.G, i_bif));
    
    parentDims = (1:numParents) + 1;
    
    if numParents > 0
        
        parents_bif = org.mensxmachina.commons.graph.parents(bnet_bif.G, i_bif);
        parents_bif_in_ordered_bif = arrayfun(@(parent_bif) find(varOrder_bif == parent_bif, 1), parents_bif);
        [~, parents_bif_in_ordered_bif_order] = sort(parents_bif_in_ordered_bif);
        parentDims_ordered_bif = parentDims(parents_bif_in_ordered_bif_order);
        
        parents_xdsl = org.mensxmachina.commons.graph.parents(bnet_xdsl.G, i_xdsl);
        parents_xdsl_in_ordered_xdsl = arrayfun(@(parent_xdsl) find(varOrder_xdsl == parent_xdsl, 1), parents_xdsl);
        [~, parents_xdsl_in_ordered_xdsl_order] = sort(parents_xdsl_in_ordered_xdsl);
        parentDims_ordered_xdsl = parentDims(parents_xdsl_in_ordered_xdsl_order);
        
        iParam_bif = permute(Param_bif{i}, [1 parentDims_ordered_bif]);
        iParam_xdsl = permute(Param_xdsl{i}, [1 parentDims_ordered_xdsl]);
        
        assertEqual(iParam_bif, iParam_xdsl);
        
    else
        
        iParam_bif = Param_bif{i};
        iParam_xdsl = Param_xdsl{i};
        assertElementsAlmostEqual(iParam_bif, iParam_xdsl);
        
    end
    
end

end

function testcmpwithdsl(obj)

import org.mensxmachina.pgm.*;

bnet_dsl = dsl2pgm(hugin2dsl('alarm_h.net'));

bnet_xdsl = xdslread('alarm.xdsl', 'nominal');

bnet_xdsl_ind_in_dsl = cellfun(@(varName) find(strcmpi(bnet_dsl.VarNames, varName), 1), bnet_xdsl.VarNames);

varOrder_dsl = bnet_xdsl_ind_in_dsl;
varOrder_xdsl = 1:length(bnet_xdsl_ind_in_dsl);

% assert equal graph
assertEqual(bnet_dsl.G(varOrder_dsl, varOrder_dsl), bnet_xdsl.G(varOrder_xdsl, varOrder_xdsl));

% assert equal varnames
assertEqual(upper(bnet_dsl.VarNames(varOrder_dsl)), bnet_xdsl.VarNames(varOrder_xdsl));

% assert equal # levels
assertEqual(bnet_dsl.NumLevels(varOrder_dsl), bnet_xdsl.NumLevels(varOrder_xdsl));

Param_dsl = bnet_dsl.Param(varOrder_dsl);
Param_xdsl = bnet_xdsl.Param(varOrder_xdsl);

% the levels of these variables are known to be permuted among the dsl and
% the xsdl files; ignore them when checking if CPTs are equal
permutedLevelsVarNames = {'LVEDVolume', 'StrokeVolume', 'InsuffAnesth', 'FiO2', 'Disconnect', 'MinVolSet', 'VentMach', 'VentTube', 'VentLung', 'VentAlv', 'PVSat', 'Shunt', 'Catechol', 'HR', 'HRBP', 'HREKG', 'HRSat', 'MinVol', 'Press', 'CO', 'BP'};

for i=1:bnet_dsl.NumNodes
    
    i_dsl = varOrder_dsl(i);
    i_xdsl = varOrder_xdsl(i);
    
    varName = bnet_dsl.VarNames{i_dsl};
    
    if ismember(varName, permutedLevelsVarNames)
        continue;
    end
    
    numParents = length(org.mensxmachina.commons.graph.parents(bnet_dsl.G, i_dsl));
    
    parentDims= (1:numParents) + 1;
    
    if numParents > 0
        
        parents_dsl = org.mensxmachina.commons.graph.parents(bnet_dsl.G, i_dsl);
        parents_dsl_in_ordered_dsl = arrayfun(@(parent_dsl) find(varOrder_dsl == parent_dsl, 1), parents_dsl);
        [~, parents_dsl_in_ordered_dsl_order] = sort(parents_dsl_in_ordered_dsl);
        parentDims_ordered_dsl = parentDims(parents_dsl_in_ordered_dsl_order);
        
        parents_xdsl = org.mensxmachina.commons.graph.parents(bnet_xdsl.G, i_xdsl);
        parents_xdsl_in_ordered_xdsl = arrayfun(@(parent_xdsl) find(varOrder_xdsl == parent_xdsl, 1), parents_xdsl);
        [~, parents_xdsl_in_ordered_xdsl_order] = sort(parents_xdsl_in_ordered_xdsl);
        parentDims_ordered_xdsl = parentDims(parents_xdsl_in_ordered_xdsl_order);

        iParam_dsl = permute(Param_dsl{i}, [1 parentDims_ordered_dsl]);
        iParam_xdsl = permute(Param_xdsl{i}, [1 parentDims_ordered_xdsl]);
        assertElementsAlmostEqual(iParam_dsl, iParam_xdsl);
        
    else
        
        iParam_dsl = Param_dsl{i};
        iParam_xdsl = Param_xdsl{i};
        assertElementsAlmostEqual(iParam_dsl, iParam_xdsl);
        
    end
    
end

end

end
    
end 
