classdef TestXmlBifRead < TestCase
%TESTXMLBIFREAD XMLBIFREAD test cases

properties
    bnet
    node
end

methods

function obj = TestXmlBifRead(name)

obj = obj@TestCase(name);

end

function testcmpwithbif(obj)

import org.mensxmachina.pgm.*;

bnet_bif = bifread('alarm.bif', 'nominal');
bnet_xmlbif = xmlbifread('alarm.xml03', 'nominal');

[~, varOrder_bif] = sort(bnet_bif.VarNames);
[~, varOrder_xmlbif] = sort(upper(bnet_xmlbif.VarNames));

G_ordered_bif = bnet_bif.G(varOrder_bif, varOrder_bif);
G_ordered_xmlbif = bnet_xmlbif.G(varOrder_xmlbif, varOrder_xmlbif);

assertEqual(G_ordered_bif, G_ordered_xmlbif);
assertEqual(bnet_bif.NumLevels(varOrder_bif), bnet_xmlbif.NumLevels(varOrder_xmlbif));

Param_bif = bnet_bif.Param(varOrder_bif);
Param_xmlbif = bnet_xmlbif.Param(varOrder_xmlbif);

% the levels of these variables are known to be permuted among the bif and
% the xml files; ignore them when checking if CPTs are equal
permutedLevelsVarNames = ...
    {'CATECHOL', 'DISCONNECT', 'EXPCO2', 'FIO2', 'HR', 'HRBP', 'HREKG', 'HRSAT', 'INSUFFANESTH', 'LVEDVOLUME', 'MINVOL', 'MINVOLSET', 'PRESS', 'PVSAT', 'STROKEVOLUME', 'VENTALV', 'VENTLUNG', 'VENTMACH', 'VENTTUBE'};
    
for i=1:bnet_bif.NumNodes
    
    i_bif = varOrder_bif(i);
    i_xmlbif = varOrder_xmlbif(i);
    
    varName = bnet_bif.VarNames{i_bif};
    
    if ismember(varName, permutedLevelsVarNames)
        continue;
    end
    
    numParents = length(org.mensxmachina.commons.graph.parents(bnet_bif.G, i_bif));
    
    parentDims = (1:numParents) + 1;
    
    if numParents > 0
        
        parents_bif = org.mensxmachina.commons.graph.parents(bnet_bif.G, i_bif);
        parents_bif_in_ordered_bif = arrayfun(@(parent_bif) find(varOrder_bif == parent_bif, 1), parents_bif);
        [~, parents_bif_in_ordered_bif_order] = sort(parents_bif_in_ordered_bif);
        parentDims_ordered_bif = parentDims(parents_bif_in_ordered_bif_order);
        
        parents_xmlbif = org.mensxmachina.commons.graph.parents(bnet_xmlbif.G, i_xmlbif);
        parents_xmlbif_in_ordered_xmlbif = arrayfun(@(parent_xmlbif) find(varOrder_xmlbif == parent_xmlbif, 1), parents_xmlbif);
        [~, parents_xmlbif_in_ordered_xmlbif_order] = sort(parents_xmlbif_in_ordered_xmlbif);
        parentDims_ordered_xmlbif = parentDims(parents_xmlbif_in_ordered_xmlbif_order);
        
        iParam_bif = permute(Param_bif{i}, [1 parentDims_ordered_bif]);
        iParam_xmlbif = permute(Param_xmlbif{i}, [1 parentDims_ordered_xmlbif]);
        
        assertEqual(iParam_bif, iParam_xmlbif);
        
    else
        
        iParam_bif = Param_bif{i};
        iParam_xmlbif = Param_xmlbif{i};
        assertElementsAlmostEqual(iParam_bif, iParam_xmlbif);
        
    end
    
end

end

function testcmpwithdsl(obj)

import org.mensxmachina.pgm.*;

bnet_dsl = dsl2pgm(hugin2dsl('alarm_h.net'));
bnet_xmlbif = xmlbifread('alarm.xml03', 'nominal');

[~, varOrder_dsl] = sort(bnet_dsl.VarNames);
[~, varOrder_xmlbif] = sort(bnet_xmlbif.VarNames);

% assert equal graph
assertEqual(bnet_dsl.G(varOrder_dsl, varOrder_dsl), bnet_xmlbif.G(varOrder_xmlbif, varOrder_xmlbif));

% assert equal varnames
assertEqual(bnet_dsl.VarNames(varOrder_dsl), bnet_xmlbif.VarNames(varOrder_xmlbif));

% assert equal # levels
assertEqual(bnet_dsl.NumLevels(varOrder_dsl), bnet_xmlbif.NumLevels(varOrder_xmlbif));

Param_dsl = bnet_dsl.Param(varOrder_dsl);
Param_xmlbif = bnet_xmlbif.Param(varOrder_xmlbif);

% the levels of these variables are known to be permuted among the dsl and
% the xsdl files; ignore them when checking if CPTs are equal
permutedLevelsVarNames = {'BP', 'CO', 'Catechol', 'ExpCO2', 'HRBP', 'HREKG', 'HRSat', 'LVEDVolume', 'MinVol', 'PVSat', 'Press', 'SaO2', 'Shunt', 'StrokeVolume', 'VentAlv', 'VentLung', 'VentTube'};

for i=1:bnet_dsl.NumNodes
    
    i_dsl = varOrder_dsl(i);
    i_xmlbif = varOrder_xmlbif(i);
    
    varName = bnet_dsl.VarNames{i_dsl};
    
    if ismember(varName, permutedLevelsVarNames)
        continue;
    end
    
    numParents = length(org.mensxmachina.commons.graph.parents(bnet_dsl.G, i_dsl));
    
    parentDims= (1:numParents) + 1;
    
    if numParents > 0
        
        parents_dsl = org.mensxmachina.commons.graph.parents(bnet_dsl.G, i_dsl);
        parents_dsl_in_ordered_dsl = arrayfun(@(parent_dsl) find(varOrder_dsl == parent_dsl, 1), parents_dsl);
        [~, parents_dsl_in_ordered_dsl_order] = sort(parents_dsl_in_ordered_dsl);
        parentDims_ordered_dsl = parentDims(parents_dsl_in_ordered_dsl_order);
        
        parents_xmlbif = org.mensxmachina.commons.graph.parents(bnet_xmlbif.G, i_xmlbif);
        parents_xmlbif_in_ordered_xmlbif = arrayfun(@(parent_xmlbif) find(varOrder_xmlbif == parent_xmlbif, 1), parents_xmlbif);
        [~, parents_xmlbif_in_ordered_xmlbif_order] = sort(parents_xmlbif_in_ordered_xmlbif);
        parentDims_ordered_xmlbif = parentDims(parents_xmlbif_in_ordered_xmlbif_order);

        iParam_dsl = permute(Param_dsl{i}, [1 parentDims_ordered_dsl]);
        iParam_xmlbif = permute(Param_xmlbif{i}, [1 parentDims_ordered_xmlbif]);
        assertElementsAlmostEqual(iParam_dsl, iParam_xmlbif);
        
    else
        
        iParam_dsl = Param_dsl{i};
        iParam_xmlbif = Param_xmlbif{i};
        assertElementsAlmostEqual(iParam_dsl, iParam_xmlbif);
        
    end
    
end

end
%{
function testcmplearn(obj)

import org.mensxmachina.pgm.*;   
    
% load alarm 
load('mxm_pgm_alarm_bnet', 'bnet');

% load alarm sample 1  
load('mxm_pgm_alarm_sample', 'a');
    
% learn skeleton
outputskeleton = mmpcskeleton(a);
    
cp = skeletonperf(bnet.skeleton, outputskeleton);
cp.Sensitivity
cp.Specificity

% load alarm from xml
bnet_xmlbif = xmlbifread('alarm.xml', 'nominal');
    
% sample it
a = bnet_xmlbif.randomds(5000);

% learn skeleton
outputskeleton_xmlbif = mmpcskeleton(a);
    
cp = skeletonperf(bnet_xmlbif.skeleton, outputskeleton_xmlbif);
cp.Sensitivity
cp.Specificity

end
%}
end
    
end 
