classdef TestBayesianNetwork < TestCase
%TESTBAYESIANNETWORK BayesianNetwork class test cases

    properties
        bnet
        node
    end

    methods

    function obj = TestBayesianNetwork(name)
        
        obj = obj@TestCase(name);
        
        % create a grass example network
        obj.bnet = org.mensxmachina.pgm.grass;
        
        obj.node = struct(...
              'Cloudy', 1, ...
              'Sprinkler', 2, ...
              'Rain', 3, ...
              'WetGrass', 4 );
        
    end
    
    function testsetdescription(obj)
        
        % test assignment of a non-string description
        assertExceptionThrown(@setbaddescription, 'org:mensxmachina:pgm:BayesianNetwork:BadDescription');
        
        function setbaddescription()
            obj.bnet.Description = -1i;
        end
        
         % test assignment of a string description
        obj.bnet.Description = 'Another description';  
        
    end
    
    function testsetclassnames(obj)
        
        % test assignment of an empty matrix
        %obj.bnet.ClassNames = [];
        %assertTrue( all(strcmp(obj.bnet.ClassNames, 'nominal')) );
        
        % test assignment of a cell array of non-strings   
        assertExceptionThrown(@setnonstrcell, 'org:mensxmachina:pgm:BayesianNetwork:BadClassNames');
        function setnonstrcell()
            obj.bnet.ClassNames = repmat(13, 1, 4);
        end     
        
        % test assignment of a cell array of strings of bad length        
        assertExceptionThrown(@setbadlength, 'org:mensxmachina:pgm:BayesianNetwork:BadClassNames');
        function setbadlength()
            obj.bnet.ClassNames = repmat({'nominal'}, 1, 5);
        end     
        
        % test assignment of a cell array of bad strings       
        assertExceptionThrown(@setbadstrings, 'org:mensxmachina:pgm:BayesianNetwork:BadClassNames');
        function setbadstrings()
            obj.bnet.ClassNames = repmat({'blah'}, 1, 4);
        end       
        
        % test assignment of a single classname
        %obj.bnet.ClassNames = 'nominal';
        %assertTrue( all(strcmp(obj.bnet.ClassNames, 'nominal')) );   
        
        % test assignment of a single bad string     
        assertExceptionThrown(@setbadstring, 'org:mensxmachina:pgm:BayesianNetwork:BadClassNames');
        function setbadstring()
            obj.bnet.ClassNames = 'blah';
        end    
        
        % test assignment of a single non-string     
        assertExceptionThrown(@setbadsingle, 'org:mensxmachina:pgm:BayesianNetwork:BadClassNames');
        function setbadsingle()
            obj.bnet.ClassNames = -1i;
        end         
        
    end
    
    function testsetvarnames(obj)
        
        % test assignment of a non-cell array of strings   
        assertExceptionThrown(@setnoncellstr, 'org:mensxmachina:pgm:BayesianNetwork:BadVarNames');
        function setnoncellstr()
            obj.bnet.VarNames = -1i;
        end
        
        % test assignment of a cell array of non-strings   
        assertExceptionThrown(@setcellnonstr, 'org:mensxmachina:pgm:BayesianNetwork:BadVarNames');
        function setcellnonstr()
            obj.bnet.VarNames = repmat({-1i}, 1, 4);
        end     
        
        % test assignment of a cell array of strings of bad length        
        assertExceptionThrown(@setbadlength, 'org:mensxmachina:pgm:BayesianNetwork:BadClassNames');
        function setbadlength()
            obj.bnet.ClassNames = {'var1', 'var2', 'var3', 'var4'};
        end      
        
        % test assignment of a cell array of strings with duplicates        
        assertExceptionThrown(@setduplicates, 'org:mensxmachina:pgm:BayesianNetwork:BadClassNames');
        function setduplicates()
            obj.bnet.ClassNames = {'var1', 'var1', 'var3', 'var4'};
        end 
        
         % test assignment of a valid cell array of strings
        %obj.bnet.ClassNames = repmat({'nominal'}, 1, 4);
        
    end
    
    function testsetlabels(obj)
        
        % test assignment of a non-cell array of strings   
        assertExceptionThrown(@setnoncellstr, 'org:mensxmachina:pgm:BayesianNetwork:BadLabels');
        function setnoncellstr()
            obj.bnet.Labels = -1i;
        end
        
        % test assignment of a cell array of non- cell array of strings   
        assertExceptionThrown(@setcellnonstr, 'org:mensxmachina:pgm:BayesianNetwork:BadLabels');
        function setcellnonstr()
            obj.bnet.Labels = repmat({-1i}, 1, 4);
        end     
        
        % test assignment of a cell array of cell array of strings
        % of lengths less than 1
        assertExceptionThrown(@setbadlength, 'org:mensxmachina:pgm:BayesianNetwork:BadLabels');
        function setbadlength()
            obj.bnet.Labels = {{}, {'false', 'true'}, {'false', 'true'}, {'false', 'true'}};
        end      
        
        % test assignment of a cell array of cell array of strings
        % with duplicates
        assertExceptionThrown(@setduplicates, 'org:mensxmachina:pgm:BayesianNetwork:BadLabels');
        function setduplicates()
            obj.bnet.Labels = {{'false'}, {'false', 'false'}, {'false', 'true'}, {'false', 'true'}};
        end     
        
        % test valid assignment
        obj.bnet.Labels = repmat({{'false', 'true'}}, 1, 4);

    end
    
    function testskeleton(obj)
        
        trueskeleton = sparse(4, 4);
        
        trueskeleton(2,1) = 1;
        trueskeleton(3,1) = 1;
        trueskeleton(1,2) = 1;
        trueskeleton(4,2) = 1;
        trueskeleton(1,3) = 1;
        trueskeleton(4,3) = 1;
        trueskeleton(2,4) = 1;
        trueskeleton(3,4) = 1;
        
        assertTrue( isequal(skeleton(obj.bnet), trueskeleton) );
        
    end
    
    function testparents(obj)
        assertTrue( isempty(parents(obj.bnet, 1)) );
        assertTrue( isequal(parents(obj.bnet, 2), 1) );
        assertTrue( isequal(parents(obj.bnet, 3), 1) );
        assertTrue( isequal(parents(obj.bnet, 4), [2; 3]) );
    end
    
    function testrandomds(obj)
        a = randomds(obj.bnet, 1000);
        skeleton = org.mensxmachina.pgm.mmpcskeleton(a);
        full(skeleton)
        full(obj.bnet.skeleton)
        assert( isequal(skeleton, obj.bnet.skeleton) );
    end
    
    end
    
end 
