classdef TestIsDSep < TestCase
%TESTORACLETESTOPT ORACLETESTOPT test cases

properties

    G
    G_prime
    ParentsInG
    ChildrenInG
    
    m
    v
    
    indEmptyC
    indEmptyB
    
    indC
    indB
    indA
    
end

methods

function obj = TestIsDSep(name)
    
import org.mensxmachina.commons.graph.*;    
    
obj = obj@TestCase(name);

obj.G = sparse(10, 10);

obj.G(1, 3) = 1;
obj.G(1, 4) = 1;
obj.G(2, 5) = 1;
obj.G(2, 6) = 1;
obj.G(3, 6) = 1;
obj.G(3, 7) = 1;
obj.G(4, 8) = 1;
obj.G(7, 9) = 1;
obj.G(8, 9) = 1;
obj.G(8, 10) = 1;

obj.m = 10;
obj.v = 1:obj.m;

% create G', the graph with links in G going in both directions
obj.G_prime = obj.G + obj.G';

% find parents in G ("inlist") and children in G ("outlist") of each node
obj.ParentsInG = arrayfun(@(j) parents(obj.G, j), obj.v, 'UniformOutput', false);
obj.ChildrenInG = arrayfun(@(j) children(obj.G, j), obj.v, 'UniformOutput', false);

%{
obj.indEmptyC = {1, 1, 3, 3, 4, 4, 7, 7, 8, 8, 9, 9, 10, 10};
obj.indEmptyB = {2, 5, 2, 5, 4, 5, 2, 5, 2, 5, 2, 5, 2, 5};

obj.indC{1} = {1, 1, 1, 1, 3, 3, 3, 4, 4, 4, 5, 6, 6, 6, 6, 7, 7};
obj.indB{1} = {6, 7, 8, 9, 4, 8, 10, 6, 10, 7, 6, 7, 8, 9, 10, 8, 10};
obj.indA{1} = {3, 3, 4, 4, 1, 1, 1, 1, 8, 1, 2, 3, 3, 3, 3, 3, 3};

obj.indC{2} = {1, 3, 4};
obj.indB{2} = {9, 9, 9};
obj.indA{2} = {[3 4], [1 7], [1 8]};
%}

end
%{
function testprev(obj)
    
G = sparse(4, 4);
G(1,2) = 1;
G(2,3) = 1;
G(4,2) = 1;
G(4,3) = 1;
 
view(biograph(G));

dsep_prev = findDSeparations(G, 3, 2) % FAIL
dsep = org.mensxmachina.pgm.isdsep(G, [], 3, 2);
find(dsep)
                
end
%}
function testtoy(obj)

%view(biograph(obj.G, {'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J'}));
   


for j1=obj.v
    
    for j2=(j1 + 1):obj.m
                
        fprintf('\nFinding dsep(%d, %d, [])...\n', j1, j2);
        
        % optimized with nonempty C and cache
        j1IsDSepWithJ2GivenJ3 = org.mensxmachina.pgm.isdsepopt(obj.G, obj.G_prime, obj.ParentsInG, obj.ChildrenInG, j1, j2, []);

        %assertEqual(j1IsDSepWithJ2GivenJ3, ~isempty( find( cellfun(@(j) j == j1, obj.indEmptyC) & cellfun(@(j) j == j2, obj.indEmptyB), 1) ) );
        
        % optimized with nonempty C and without cache
        j1IsDSepWithJ2GivenJ3_2 = org.mensxmachina.pgm.isdsepopt(obj.G, obj.G_prime, obj.ParentsInG, obj.ChildrenInG, j1, j2, []);
        assertEqual(j1IsDSepWithJ2GivenJ3_2, j1IsDSepWithJ2GivenJ3);

        % optimized with empty C and cache
        isDSepWithJ2GivenJ3 = org.mensxmachina.pgm.isdsepopt(obj.G, obj.G_prime, obj.ParentsInG, obj.ChildrenInG, [], j2, []);
        assertEqual(isDSepWithJ2GivenJ3(j1), j1IsDSepWithJ2GivenJ3);

        % optimized with empty C and without cache
        [isDSepWithJ2GivenJ3] = org.mensxmachina.pgm.isdsepopt(obj.G, obj.G_prime, obj.ParentsInG, obj.ChildrenInG, [], j2, []);
        assertEqual(isDSepWithJ2GivenJ3(j1), j1IsDSepWithJ2GivenJ3);

        % non-optimized with nonempty C
        j1IsDSepWithJ2GivenJ3_2 = org.mensxmachina.pgm.isdsep(obj.G, j1, j2, []);
        assertEqual(j1IsDSepWithJ2GivenJ3_2, j1IsDSepWithJ2GivenJ3);

        % non-optimized with empty C
        isDSepWithJ2GivenJ3 = org.mensxmachina.pgm.isdsep(obj.G, [], j2, []);
        assertEqual(isDSepWithJ2GivenJ3(j1), j1IsDSepWithJ2GivenJ3);

        for nj3 = 1:2 % for each cardinality up to 2

            C3 = nchoosek(setdiff(obj.v, j2), nj3);

            for k3 = 1:size(C3, 1) % for each subset of this cardinality

                j3 = C3(k3, :);
                
                fprintf('\nFinding dsep(%d, %d, %s)...\n', j1, j2, num2str(j3));
                
                % optimized with nonempty C and cache
                j1IsDSepWithJ2GivenJ3 = org.mensxmachina.pgm.isdsepopt(obj.G, obj.G_prime, obj.ParentsInG, obj.ChildrenInG, j1, j2, j3);
                
                %assertEqual(j1IsDSepWithJ2GivenJ3, ~isempty( find( cellfun(@(j) j == j1, obj.indC{nj3}) & cellfun(@(j) j == j2, obj.indB{nj3}) & cellfun(@(j) j == j3, obj.indA{nj3}),  1) ) );
        
                % optimized with nonempty C and without cache
                j1IsDSepWithJ2GivenJ3_2 = org.mensxmachina.pgm.isdsepopt(obj.G, obj.G_prime, obj.ParentsInG, obj.ChildrenInG, j1, j2, j3);
                assertEqual(j1IsDSepWithJ2GivenJ3_2, j1IsDSepWithJ2GivenJ3);
                
                % optimized with empty C and cache
                isDSepWithJ2GivenJ3 = org.mensxmachina.pgm.isdsepopt(obj.G, obj.G_prime, obj.ParentsInG, obj.ChildrenInG, [], j2, j3);
                assertEqual(isDSepWithJ2GivenJ3(j1), j1IsDSepWithJ2GivenJ3);
                
                % optimized with empty C and without cache
                [isDSepWithJ2GivenJ3] = org.mensxmachina.pgm.isdsepopt(obj.G, obj.G_prime, obj.ParentsInG, obj.ChildrenInG, [], j2, j3);
                assertEqual(isDSepWithJ2GivenJ3(j1), j1IsDSepWithJ2GivenJ3);
                
                % non-optimized with nonempty C
                j1IsDSepWithJ2GivenJ3_2 = org.mensxmachina.pgm.isdsep(obj.G, j1, j2, j3);
                assertEqual(j1IsDSepWithJ2GivenJ3_2, j1IsDSepWithJ2GivenJ3);
                
                % non-optimized with empty C
                isDSepWithJ2GivenJ3 = org.mensxmachina.pgm.isdsep(obj.G, [], j2, j3);
                assertEqual(isDSepWithJ2GivenJ3(j1), j1IsDSepWithJ2GivenJ3);

            end

        end
    
    end

end

end
%{
function testwithoraclepcskeleton(obj)

import org.mensxmachina.commons.graph.*;
import org.mensxmachina.commons.stats.*;
import org.mensxmachina.pgm.*;    
    
bnetName = 'alarm';    
sampleNo = 1;    

% get bayesian network graph
load( sprintf('mxm_pgm_%s_bnet', bnetName), 'bnet');
G = bnet.G;
clear bnet;    
    
a = [];

% load sample
load(sprintf('mxm_pgm_%s_sample_%d', bnetName, sampleNo), 'a');

v = 1:size(a, 2);

% limit the sample (doesn't matter)
a_ss = a(1:10, :);

PParam = struct( ...
    'G', G, ...
    'GPrime', G + G', ...
    'ParentsInG', {arrayfun(@(j) parents(G, j), v, 'UniformOutput', false)}, ...
    'ChildrenInG', {arrayfun(@(j) children(G, j), v, 'UniformOutput', false)} ...
    );

% end create oraclepcskeleton parameters

disp('Learning skeleton with oraclepcskeleton...');

skeleton = oraclepcskeleton( a_ss, G, 'PowerFun', @dummyindtestpwropt, 'PowerMaxKFun', @dummyindtestpwrmaxkopt, 'PFun', @oracleindtestpopt, 'PParam', PParam);

assertEqual(skeleton, org.mensxmachina.commons.graph.skeleton(G));

end
%}
end

end